@@ -794,6 +794,14 @@ int pa_alsa_ucm_query_profiles(pa_alsa_ucm_config *ucm, int card_index) {
 
     pa_log_info("UCM available for card %s", card_name);
 
+    if (snd_use_case_get(ucm->ucm_mgr, "_alibpref", &value) == 0) {
+        if (value[0]) {
+            ucm->alib_prefix = pa_xstrdup(value);
+            pa_log_debug("UCM _alibpref=%s", ucm->alib_prefix);
+        }
+        free((void *)value);
+    }
+
     /* get a list of all UCM verbs (profiles) for this card */
     num_verbs = snd_use_case_verb_list(ucm->ucm_mgr, &verb_list);
     if (num_verbs < 0) {
@@ -931,6 +939,19 @@ static void set_eld_devices(pa_hashmap *hash)
     }
 }
 
+static void update_mixer_paths(pa_hashmap *ports, const char *profile) {
+    pa_device_port *port;
+    pa_alsa_ucm_port_data *data;
+    void *state;
+
+    /* select volume controls on ports */
+    PA_HASHMAP_FOREACH(port, ports, state) {
+        pa_log_info("Updating mixer path for %s: %s", profile, port->name);
+        data = PA_DEVICE_PORT_DATA(port);
+        data->path = pa_hashmap_get(data->paths, profile);
+    }
+}
+
 static void probe_volumes(pa_hashmap *hash, bool is_sink, snd_pcm_t *pcm_handle, pa_hashmap *mixers, bool ignore_dB) {
     pa_device_port *port;
     pa_alsa_path *path;
@@ -964,11 +985,12 @@ static void probe_volumes(pa_hashmap *hash, bool is_sink, snd_pcm_t *pcm_handle,
             if (pa_alsa_path_probe(path, NULL, mixer_handle, ignore_dB) < 0) {
                 pa_log_warn("Could not probe path: %s, using s/w volume", path->name);
                 pa_hashmap_remove(data->paths, profile);
-            } else if (!path->has_volume) {
-                pa_log_warn("Path %s is not a volume control", path->name);
+            } else if (!path->has_volume && !path->has_mute) {
+                pa_log_warn("Path %s is not a volume or mute control", path->name);
                 pa_hashmap_remove(data->paths, profile);
             } else
-                pa_log_debug("Set up h/w volume using '%s' for %s:%s", path->name, profile, port->name);
+                pa_log_debug("Set up h/w %s using '%s' for %s:%s", path->has_volume ? "volume" : "mute",
+                                path->name, profile, port->name);
         }
     }
 
@@ -1092,8 +1114,6 @@ static void ucm_add_port_combination(
         if (num == 1) {
             /* To keep things simple and not worry about stacking controls, we only support hardware volumes on non-combination
              * ports. */
-            data = PA_DEVICE_PORT_DATA(port);
-
             PA_HASHMAP_FOREACH_KV(profile, vol, is_sink ? dev->playback_volumes : dev->capture_volumes, state) {
                 pa_alsa_path *path = pa_alsa_path_synthesize(vol->mixer_elem,
                                                              is_sink ? PA_ALSA_DIRECTION_OUTPUT : PA_ALSA_DIRECTION_INPUT);
@@ -1322,6 +1342,14 @@ void pa_alsa_ucm_add_ports(
     /* now set up volume paths if any */
     probe_volumes(*p, is_sink, pcm_handle, context->ucm->mixers, ignore_dB);
 
+    /* probe_volumes() removes per-profile paths from ports if probing them
+     * fails. The path for the current profile is cached in
+     * pa_alsa_ucm_port_data.path, which is not cleared by probe_volumes() if
+     * the path gets removed, so we have to call update_mixer_paths() here to
+     * unset the cached path if needed. */
+    if (card->active_profile)
+        update_mixer_paths(*p, card->active_profile->name);
+
     /* then set property PA_PROP_DEVICE_INTENDED_ROLES */
     merged_roles = pa_xstrdup(pa_proplist_gets(proplist, PA_PROP_DEVICE_INTENDED_ROLES));
     PA_IDXSET_FOREACH(dev, context->ucm_devices, idx) {
@@ -1350,9 +1378,6 @@ int pa_alsa_ucm_set_profile(pa_alsa_ucm_config *ucm, pa_card *card, const char *
     int ret = 0;
     const char *profile;
     pa_alsa_ucm_verb *verb;
-    pa_device_port *port;
-    pa_alsa_ucm_port_data *data;
-    void *state;
 
     if (new_profile == old_profile)
         return ret;
@@ -1381,12 +1406,7 @@ int pa_alsa_ucm_set_profile(pa_alsa_ucm_config *ucm, pa_card *card, const char *
         }
     }
 
-    /* select volume controls on ports */
-    PA_HASHMAP_FOREACH(port, card->ports, state) {
-        data = PA_DEVICE_PORT_DATA(port);
-        data->path = pa_hashmap_get(data->paths, profile);
-    }
-
+    update_mixer_paths(card->ports, profile);
     return ret;
 }
 
@@ -1542,15 +1562,11 @@ static pa_alsa_mapping* ucm_alsa_mapping_get(pa_alsa_ucm_config *ucm, pa_alsa_pr
     pa_alsa_mapping *m;
     char *mapping_name;
     size_t ucm_alibpref_len = 0;
-    const char *value;
 
     /* find private alsa-lib's configuration device prefix */
-    if (snd_use_case_get(ucm->ucm_mgr, "_alibpref", &value) == 0) {
-        if (value[0] && pa_startswith(device_str, value))
-            ucm_alibpref_len = strlen(value);
 
-        free((void *)value);
-    }
+    if (ucm->alib_prefix && pa_startswith(device_str, ucm->alib_prefix))
+        ucm_alibpref_len = strlen(ucm->alib_prefix);
 
     mapping_name = pa_sprintf_malloc("Mapping %s: %s: %s", verb_name, device_str + ucm_alibpref_len, is_sink ? "sink" : "source");
 
@@ -1872,7 +1888,7 @@ static void mapping_init_eld(pa_alsa_mapping *m, snd_pcm_t *pcm)
     pa_alsa_ucm_mapping_context *context = &m->ucm_context;
     pa_alsa_ucm_device *dev;
     uint32_t idx;
-    char *mdev;
+    char *mdev, *alib_prefix;
     snd_pcm_info_t *info;
     int pcm_card, pcm_device;
 
@@ -1885,8 +1901,10 @@ static void mapping_init_eld(pa_alsa_mapping *m, snd_pcm_t *pcm)
     if ((pcm_device = snd_pcm_info_get_device(info)) < 0)
         return;
 
+    alib_prefix = context->ucm->alib_prefix;
+
     PA_IDXSET_FOREACH(dev, context->ucm_devices, idx) {
-       mdev = pa_sprintf_malloc("hw:%i", pcm_card);
+       mdev = pa_sprintf_malloc("%shw:%i", alib_prefix ? alib_prefix : "", pcm_card);
        if (mdev == NULL)
            continue;
        dev->eld_mixer_device_name = mdev;
@@ -2152,6 +2170,8 @@ void pa_alsa_ucm_free(pa_alsa_ucm_config *ucm) {
         snd_use_case_mgr_close(ucm->ucm_mgr);
         ucm->ucm_mgr = NULL;
     }
+    pa_xfree(ucm->alib_prefix);
+    ucm->alib_prefix = NULL;
 }
 
 void pa_alsa_ucm_mapping_context_free(pa_alsa_ucm_mapping_context *context) {
