@@ -18,9 +18,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #include <inttypes.h>
@@ -173,11 +171,45 @@
  * \li Sink input - pa_context_kill_sink_input()
  * \li Source output - pa_context_kill_source_output()
  *
+ * It is strongly recommended that all volume changes are done as a direct
+ * result of user input. With automated requests, such as those resulting
+ * from misguided attempts of crossfading, PulseAudio can store the stream
+ * volume at an inappropriate moment and restore it later. Besides, such
+ * attempts lead to OSD popups in some desktop environments.
+ *
+ * As a special case of the general rule above, it is recommended that your
+ * application leaves the task of saving and restoring the volume of its
+ * streams to PulseAudio and does not attempt to do it by itself. PulseAudio
+ * really knows better about events such as stream moving or headphone
+ * plugging that would make the volume stored by the application inapplicable
+ * to the new configuration.
+ *
+ * Another important case where setting a sink input volume may be a bad idea
+ * is related to interpreters that interpret potentially untrusted scripts.
+ * PulseAudio relies on your application not making malicious requests (such
+ * as repeatedly setting the volume to 100%). Thus, script interpreters that
+ * represent a security boundary must sandbox volume-changing requests coming
+ * from their scripts. In the worst case, it may be necessary to apply the
+ * script-requested volume to the script-produced sounds by altering the
+ * samples in the script interpreter and not touching the sink or sink input
+ * volume as seen by PulseAudio.
+ *
+ * If an application changes any volume, it should also listen to changes of
+ * the same volume originating from outside the application (e.g., from the
+ * system mixer application) and update its user interface accordingly. Use
+ * \ref subscribe to get such notifications.
+ *
  * \subsection module_subsec Modules
  *
  * Server modules can be remotely loaded and unloaded using
  * pa_context_load_module() and pa_context_unload_module().
  *
+ * \subsection message_subsec Messages
+ *
+ * Server objects like sinks, sink inputs or modules can register a message
+ * handler to communicate with clients. A message can be sent to a named
+ * message handler using pa_context_send_message_to_object().
+ *
  * \subsection client_subsec Clients
  *
  * The only operation supported on clients is the possibility of kicking
@@ -203,6 +235,30 @@ typedef struct pa_sink_port_info {
     const char *description;            /**< Description of this port */
     uint32_t priority;                  /**< The higher this value is, the more useful this port is as a default. */
     int available;                      /**< A flags (see #pa_port_available), indicating availability status of this port. \since 2.0 */
+    const char *availability_group;     /**< An indentifier for the group of ports that share their availability status with
+                                         * each other. This is meant especially for handling cases where one 3.5 mm connector
+                                         * is used for headphones, headsets and microphones, and the hardware can only tell
+                                         * that something was plugged in but not what exactly. In this situation the ports for
+                                         * all those devices share their availability status, and PulseAudio can't tell which
+                                         * one is actually plugged in, and some application may ask the user what was plugged
+                                         * in. Such applications should get a list of all card ports and compare their
+                                         * `availability_group` fields. Ports that have the same group are those that need
+                                         * input from the user to determine which device was plugged in. The application should
+                                         * then activate the user-chosen port.
+                                         *
+                                         * May be NULL, in which case the port is not part of any availability group.
+                                         *
+                                         * The group identifier must be treated as an opaque identifier. The string may look
+                                         * like an ALSA control name, but applications must not assume any such relationship.
+                                         * The group naming scheme can change without a warning.
+                                         *
+                                         * Since one group can include both input and output ports, the grouping should be done
+                                         * using pa_card_port_info instead of pa_sink_port_info, but this field is duplicated
+                                         * also in pa_sink_port_info (and pa_source_port_info) in case someone finds that
+                                         * convenient.
+                                         *
+                                         * \since 14.0 */
+    uint32_t type;			/**< Port type, see #pa_device_port_type. \since 14.0 */
 } pa_sink_port_info;
 
 /** Stores information about sinks. Please note that this structure
@@ -283,6 +339,31 @@ typedef struct pa_source_port_info {
     const char *description;            /**< Description of this port */
     uint32_t priority;                  /**< The higher this value is, the more useful this port is as a default. */
     int available;                      /**< A flags (see #pa_port_available), indicating availability status of this port. \since 2.0 */
+    const char *availability_group;     /**< An indentifier for the group of ports that share their availability status with
+                                         * each other. This is meant especially for handling cases where one 3.5 mm connector
+                                         * is used for headphones, headsets and microphones, and the hardware can only tell
+                                         * that something was plugged in but not what exactly. In this situation the ports for
+                                         * all those devices share their availability status, and PulseAudio can't tell which
+                                         * one is actually plugged in, and some application may ask the user what was plugged
+                                         * in. Such applications should get a list of all card ports and compare their
+                                         * `availability_group` fields. Ports that have the same group are those that need
+                                         * input from the user to determine which device was plugged in. The application should
+                                         * then activate the user-chosen port.
+                                         *
+                                         * May be NULL, in which case the port is not part of any availability group (which is
+                                         * the same as having a group with only one member).
+                                         *
+                                         * The group identifier must be treated as an opaque identifier. The string may look
+                                         * like an ALSA control name, but applications must not assume any such relationship.
+                                         * The group naming scheme can change without a warning.
+                                         *
+                                         * Since one group can include both input and output ports, the grouping should be done
+                                         * using pa_card_port_info instead of pa_source_port_info, but this field is duplicated
+                                         * also in pa_source_port_info (and pa_sink_port_info) in case someone finds that
+                                         * convenient.
+                                         *
+                                         * \since 14.0 */
+    uint32_t type;		        /**< Port type, see #pa_device_port_type. \since 14.0 */
 } pa_source_port_info;
 
 /** Stores information about sources. Please note that this structure
@@ -298,7 +379,7 @@ typedef struct pa_source_info {
     pa_cvolume volume;                  /**< Volume of the source */
     int mute;                           /**< Mute switch of the sink */
     uint32_t monitor_of_sink;           /**< If this is a monitor source, the index of the owning sink, otherwise PA_INVALID_INDEX. */
-    const char *monitor_of_sink_name;   /**< Name of the owning sink, or PA_INVALID_INDEX. */
+    const char *monitor_of_sink_name;   /**< Name of the owning sink, or NULL. */
     pa_usec_t latency;                  /**< Length of filled record buffer of this source. */
     const char *driver;                 /**< Driver name */
     pa_source_flags_t flags;            /**< Flags */
@@ -414,6 +495,17 @@ pa_operation* pa_context_unload_module(pa_context *c, uint32_t idx, pa_context_s
 
 /** @} */
 
+/** @{ \name Messages */
+
+/** Callback prototype for pa_context_send_message_to_object() \since 15.0 */
+typedef void (*pa_context_string_cb_t)(pa_context *c, int success, char *response, void *userdata);
+
+/** Send a message to an object that registered a message handler. For more information
+ * see https://cgit.freedesktop.org/pulseaudio/pulseaudio/tree/doc/messaging_api.txt. \since 15.0 */
+pa_operation* pa_context_send_message_to_object(pa_context *c, const char *recipient_name, const char *message, const char *message_parameters, pa_context_string_cb_t cb, void *userdata);
+
+/** @} */
+
 /** @{ \name Clients */
 
 /** Stores information about clients. Please note that this structure
@@ -443,9 +535,7 @@ pa_operation* pa_context_kill_client(pa_context *c, uint32_t idx, pa_context_suc
 
 /** @{ \name Cards */
 
-/** Stores information about a specific profile of a card.  Please
- * note that this structure can be extended as part of evolutionary
- * API updates at any time in any new release. \since 0.9.15 */
+/** \deprecated Superseded by pa_card_profile_info2 \since 0.9.15 */
 typedef struct pa_card_profile_info {
     const char *name;                   /**< Name of this profile */
     const char *description;            /**< Description of this profile */
@@ -454,6 +544,23 @@ typedef struct pa_card_profile_info {
     uint32_t priority;                  /**< The higher this value is, the more useful this profile is as a default. */
 } pa_card_profile_info;
 
+/** Stores information about a specific profile of a card. Please
+ * note that this structure can be extended as part of evolutionary
+ * API updates at any time in any new release. \since 5.0 */
+typedef struct pa_card_profile_info2 {
+    const char *name;                   /**< Name of this profile */
+    const char *description;            /**< Description of this profile */
+    uint32_t n_sinks;                   /**< Number of sinks this profile would create */
+    uint32_t n_sources;                 /**< Number of sources this profile would create */
+    uint32_t priority;                  /**< The higher this value is, the more useful this profile is as a default. */
+    int available;
+    /**< Is this profile available? If this is zero, meaning "unavailable",
+     * then it makes no sense to try to activate this profile. If this is
+     * non-zero, it's still not a guarantee that activating the profile will
+     * result in anything useful, it just means that the server isn't aware of
+     * any reason why the profile would definitely be useless. \since 5.0 */
+} pa_card_profile_info2;
+
 /** Stores information about a specific port of a card.  Please
  * note that this structure can be extended as part of evolutionary
  * API updates at any time in any new release. \since 2.0 */
@@ -464,9 +571,30 @@ typedef struct pa_card_port_info {
     int available;                      /**< A #pa_port_available enum, indicating availability status of this port. */
     int direction;                      /**< A #pa_direction enum, indicating the direction of this port. */
     uint32_t n_profiles;                /**< Number of entries in profile array */
-    pa_card_profile_info** profiles;    /**< Array of pointers to available profiles, or NULL. Array is terminated by an entry set to NULL. */
+    pa_card_profile_info** profiles;    /**< \deprecated Superseded by profiles2 */
     pa_proplist *proplist;              /**< Property list */
     int64_t latency_offset;             /**< Latency offset of the port that gets added to the sink/source latency when the port is active. \since 3.0 */
+    pa_card_profile_info2** profiles2;  /**< Array of pointers to available profiles, or NULL. Array is terminated by an entry set to NULL. \since 5.0 */
+    const char *availability_group;     /**< An indentifier for the group of ports that share their availability status with
+                                         * each other. This is meant especially for handling cases where one 3.5 mm connector
+                                         * is used for headphones, headsets and microphones, and the hardware can only tell
+                                         * that something was plugged in but not what exactly. In this situation the ports for
+                                         * all those devices share their availability status, and PulseAudio can't tell which
+                                         * one is actually plugged in, and some application may ask the user what was plugged
+                                         * in. Such applications should get a list of all card ports and compare their
+                                         * `availability_group` fields. Ports that have the same group are those that need
+                                         * input from the user to determine which device was plugged in. The application should
+                                         * then activate the user-chosen port.
+                                         *
+                                         * May be NULL, in which case the port is not part of any availability group (which is
+                                         * the same as having a group with only one member).
+                                         *
+                                         * The group identifier must be treated as an opaque identifier. The string may look
+                                         * like an ALSA control name, but applications must not assume any such relationship.
+                                         * The group naming scheme can change without a warning.
+                                         *
+                                         * \since 14.0 */
+    uint32_t type;			/**< Port type, see #pa_device_port_type. \since 14.0 */
 } pa_card_port_info;
 
 /** Stores information about cards. Please note that this structure
@@ -478,11 +606,13 @@ typedef struct pa_card_info {
     uint32_t owner_module;               /**< Index of the owning module, or PA_INVALID_INDEX. */
     const char *driver;                  /**< Driver name */
     uint32_t n_profiles;                 /**< Number of entries in profile array */
-    pa_card_profile_info* profiles;      /**< Array of available profile, or NULL. Array is terminated by an entry with name set to NULL. Number of entries is stored in n_profiles. */
-    pa_card_profile_info* active_profile; /**< Pointer to active profile in the array, or NULL. */
+    pa_card_profile_info* profiles;      /**< \deprecated Superseded by profiles2 */
+    pa_card_profile_info* active_profile; /**< \deprecated Superseded by active_profile2 */
     pa_proplist *proplist;               /**< Property list */
     uint32_t n_ports;                    /**< Number of entries in port array */
     pa_card_port_info **ports;           /**< Array of pointers to ports, or NULL. Array is terminated by an entry set to NULL. */
+    pa_card_profile_info2** profiles2;    /**< Array of pointers to available profiles, or NULL. Array is terminated by an entry set to NULL. \since 5.0 */
+    pa_card_profile_info2* active_profile2; /**< Pointer to active profile in the array, or NULL. \since 5.0 */
 } pa_card_info;
 
 /** Callback prototype for pa_context_get_card_info_...() \since 0.9.15 */
