@@ -681,7 +681,7 @@ static void help(const char *argv0) {
              "  -p, --playback                        Create a connection for playback\n\n"
              "  -v, --verbose                         Enable verbose operations\n\n"
              "  -s, --server=SERVER                   The name of the server to connect to\n"
-             "  -d, --device=DEVICE                   The name of the sink/source to connect to\n"
+             "  -d, --device=DEVICE                   The name of the sink/source to connect to. The special names @DEFAULT_SINK@, @DEFAULT_SOURCE@ and @DEFAULT_MONITOR@ can be used to specify the default sink, source and monitor respectively.\n"
              "  -n, --client-name=NAME                How to call this client on the server\n"
              "      --stream-name=NAME                How to call this stream on the server\n"
              "      --volume=VOLUME                   Specify the initial (linear) volume in range 0...65536\n"
